/*
 * This file is part of Hopsworks
 * Copyright (C) 2018, Logical Clocks AB. All rights reserved
 *
 * Hopsworks is free software: you can redistribute it and/or modify it under the terms of
 * the GNU Affero General Public License as published by the Free Software Foundation,
 * either version 3 of the License, or (at your option) any later version.
 *
 * Hopsworks is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License along with this program.
 * If not, see <https://www.gnu.org/licenses/>.
 */
package io.hops.hopsworks.persistence.entity.jwt;

import java.io.Serializable;
import java.util.Date;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.xml.bind.annotation.XmlRootElement;

@Entity
@Table(name = "jwt_signing_key", catalog = "hopsworks")
@XmlRootElement
@NamedQueries({
  @NamedQuery(name = "JwtSigningKey.findAll",
      query = "SELECT j FROM JwtSigningKey j")
  ,
    @NamedQuery(name = "JwtSigningKey.findById",
      query = "SELECT j FROM JwtSigningKey j WHERE j.id = :id")
  ,
    @NamedQuery(name = "JwtSigningKey.findBySecret",
      query = "SELECT j FROM JwtSigningKey j WHERE j.secret = :secret")
  ,
    @NamedQuery(name = "JwtSigningKey.findByName",
      query = "SELECT j FROM JwtSigningKey j WHERE j.name = :name")
  ,
    @NamedQuery(name = "JwtSigningKey.findByCreatedOn",
      query
      = "SELECT j FROM JwtSigningKey j WHERE j.createdOn = :createdOn")})
public class JwtSigningKey implements Serializable {

  private static final long serialVersionUID = 1L;

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  @Basic(optional = false)
  @Column(name = "id")
  private Integer id;

  @Basic(optional = false)
  @NotNull
  @Size(min = 1,
      max = 128)
  @Column(name = "secret")
  private String secret;

  @Basic(optional = false)
  @NotNull
  @Size(min = 1,
      max = 45)
  @Column(name = "name")
  private String name;

  @Basic(optional = true)
  @Column(name = "created_on", updatable= false, insertable=false)
  @Temporal(TemporalType.TIMESTAMP)
  private Date createdOn;

  public JwtSigningKey() {
  }

  public JwtSigningKey(Integer id) {
    this.id = id;
  }

  public JwtSigningKey(String secret, String name) {
    this.secret = secret;
    this.name = name;
//    this.createdOn = java.sql.Date.valueOf(java.time.LocalDate.now());
  }

  public Integer getId() {
    return id;
  }

  public void setId(Integer id) {
    this.id = id;
  }

  public String getSecret() {
    return secret;
  }

  public void setSecret(String secret) {
    this.secret = secret;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public Date getCreatedOn() {
    return createdOn;
  }

  public void setCreatedOn(Date createdOn) {
    this.createdOn = createdOn;
  }

  @Override
  public int hashCode() {
    int hash = 0;
    hash += (id != null ? id.hashCode() : 0);
    return hash;
  }

  @Override
  public boolean equals(Object object) {
    // TODO: Warning - this method won't work in the case the id fields are not set
    if (!(object instanceof JwtSigningKey)) {
      return false;
    }
    JwtSigningKey other = (JwtSigningKey) object;
    if ((this.id == null && other.id != null) || (this.id != null && !this.id.equals(other.id))) {
      return false;
    }
    return true;
  }

  @Override
  public String toString() {
    return "io.hops.hopsworks.persistence.entity.jwt.JwtSigningKey[ id=" + id + " ]";
  }
  
}
